package com.mycompany.errordemo;

import javax.ejb.Stateless;

//
// Injected fine with both @EJB and @Inject. However, if @Inject
// is used, triggering use of CDI, the JavaAssist wrapper that CDI generates
// will try to use InheritParent rather than InheritFailsEJB when the getValue()
// method is called. This throws an exception:
//
//   java.lang.IllegalStateException: Unable to convert ejbRef for ejb InheritFailsEJB to a business object of type class com.mycompany.errordemo.InheritParent
//
// If the @Stateless annotation is removed from this class, turning it into a regular
// bean not a managed EJB, then CDI will @Inject it without problems, and will successfully
// call its inherited methods. So it looks like the core problem is that CDI/Weld won't resolve
// inherited methods on local no-interface views of EJBs, though it does so fine on unmanaged beans.
//
// Bizarrely, under JBoss AS 6 CDI/Weld does this fine, but fails to deal with the JSF2 code...
//
@Stateless
public class InheritFailsEJB extends InheritParent {

    // getValue() isn't overridden here. That's what seems to confuse CDI / JavaAssist.
    // If you override getValue(), the problem will go away because CDI can resolve
    // the method without having to access the superclass.

    @Override
    public String getOtherValue() {
        return "InheritFailsEJB";
    }
    
}