package com.mycompany.errordemo;

import java.io.Serializable;
import javax.inject.Inject;

/**
 * This backing bean uses CDI/JSF2 integration for its lifetime management
 * and EL naming. It uses CDI for dependency injection.
 *
 * This bean demonstrates how CDI's dynamic method resolution is causing issues
 * where a method of a superclass is accessed via a local no-interface view.
 *
 * This bean is session scoped (and therefore must be serializable) because
 * that's a requirement in the real situation this test case was extracted from.
 * If we could use a RequestScoped bean here, we could just switch from injecting
 * an EJB to injecting an ordinary bean, since CDI's issues seem to be specific
 * to local no-interface views of EJBs.
 *
 * @author craig
 */
@javax.inject.Named
@javax.enterprise.context.SessionScoped
public class RequestBeanCDI implements Serializable {

    private static final long serialVersionUID = 1L;

    // If @EJB is used instead of @Inject (avoiding CDI+JavaAssist injection)
    // or if @Inject ios used with InheritOKEJB instead of InheritFailsEJB, the fault goes away.
    @Inject private InheritFailsEJB backingEJB;

    public String getValue() {
        try {
            DebugHelper.logObject("CDI getValue", backingEJB);
            // We successfully injected our InheritFailsEJB with @Inject.
            // However, the following call will throw a java.lang.IllegalStateException.
            // See InheritFailsEJB JavaDoc.
            return backingEJB.getValue();
        } catch (IllegalStateException ex) {
            ex.printStackTrace(System.err);
            return "(FAILED) " + ex.toString();
        }
    }

    public String getOtherValue() {
        try {
            DebugHelper.logObject("CDI getOtherValue", backingEJB);
            return backingEJB.getOtherValue();
        } catch (IllegalStateException ex) {
            ex.printStackTrace(System.err);
            return "(FAILED) " + ex.toString();
        }
    }
}